/*
 * Vwap.java
 *
 * This sample java monitor is used to compute Volume Weighted Average Price (VWAP) 
 * data for given stocks and emit an event when consecutive VWAP calculations 
 * differ by a given percentage. It makes use of Timer events, created outside the 
 * java monitor, to monitor the passing of time and mark when a VWAP needs to be
 * calculated.
 *
 * $Copyright(c) 2002,  2004 - 2006 Progress Software Corporation. All Rights Reserved.$ 
 * $Copyright (c) 2013,2019 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$ 
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG 
 *
 * $Revision: 359265 $
 */

import com.apama.jmon.*;


public class Vwap implements Monitor {
	private static final long serialVersionUID = 24L;
    // the vwap watch event used to instantiation a vwap worker
    private VwapWatch watch;
    
    // the time in seconds into the period over which to calculate a VWAP
    private long periodCount = 0;

    // flag to denote if the monitor is actively accumulating vwap information
    private boolean active = false;

    // variables used to accumulate information about a stock trade vwap 
    private double cumulativePV = 0.0;
    private double cumulativeV = 0.0;
    private double lastVwap = -1.0;
    private double newVwap = 0.0;
    private double change;
    
    // the event expressions and match listeners used in the class instantiation
    private EventExpression timeExpr = null;
    private EventExpression dieExpr = null;
    private EventExpression tradeExpr = null;
    private TimeMatchListener timer; 
    private DieMatchListener killer;
    private CalculateMatchListener calculate;
    


    /**
     * Inner class to respond to timer events. Sets the object to be active on the 
     * receipt of the first timer event, then increments the periodCount variable on 
     * all further cases. Calls the periodExpired method every time the duration to 
     * calculate the VWAP over expires
     *
     */
    class TimeMatchListener implements MatchListener {
	private static final long serialVersionUID = 43L;
	public void match(MatchEvent event) {
	    if (!active) active = true;
	    periodCount++;
	    if (periodCount == watch.period){
		periodCount = 0;
		periodExpired();
	    }
	}
    }


 
    /** 
     * Inner class to respond to vwap watch events for the same stock name. Removes
     * all match listeners from the event expressions set up so that the object can
     * be marked for garbage collection
     */
    class DieMatchListener implements MatchListener {
		private static final long serialVersionUID = 40L;
        public void match(MatchEvent event) {
	    if (timeExpr != null) timeExpr.removeMatchListener(timer);
	    if (dieExpr != null) dieExpr.removeMatchListener(killer);
	    if (tradeExpr != null) tradeExpr.removeMatchListener(calculate);
        }
    }
  


    /** 
     * Inner class to respond to trade events from the same stock name. Accumulates
     * the price*volume, and volume information for the trade. These values are re-set
     * every time the VWAP duration expires
     */
    class CalculateMatchListener implements MatchListener {
	private static final long serialVersionUID = 41L;
	public void match(MatchEvent event) {
	    if (active){
		Trade trade = (Trade)event.getMatchingEvents().get("trade");
		cumulativePV = cumulativePV + (trade.price * trade.volume);
		cumulativeV =  cumulativeV + trade.volume;
	    }
	}
    }



    /**
     * No argument constructor used by the jmon framework on application loading. 
     * Objects instantiated with this constructor become top-level or factory monitors, 
     * looking for vwap watch events to denote a request to start calculating VWAPs
     * for a given stock.
     */
    public Vwap() {}



    /**
     * Construct an instantiation of the class to repetitively calculate the 
     * VWAP of a given stock trade, over set periods. The constructor uses
     * the vwap watch event to denote the stock name to monitor, and the duration
     * used to calcualte the VWAP. The constructor sets up event expressions and 
     * match listeners to record timing information, to mark itself for garbage 
     * collection when a vwap watch event is seen for the same trade, and to monitor
     * all trades for the stock in question.
     *
     * @param watch  The vwap watch event used to start calculation of 
     *               VWAPs for a given stock trade
     */
    public Vwap(VwapWatch watch) {
      	this.watch = watch;

	// set up an event expression and match listener to capture timer
	// events, denoting the time in seconds since instantiation
	timer = new TimeMatchListener();
	timeExpr = new EventExpression("all Timer()");
	timeExpr.addMatchListener(timer); 


	// set up event expression and match listener to effectively remove
	// this object when another vwap watch event is seen for this stock symbol
	killer = new DieMatchListener();
	dieExpr = new EventExpression("com.apama.samples.vwap.VwapWatch(\"" + watch.stockName + "\", *, *)");
	dieExpr.addMatchListener(killer); 


	// set up event expression and match listener to accummulate and calculate
	// price/volume information for this stock
	calculate = new CalculateMatchListener();
	tradeExpr = new EventExpression("all com.apama.samples.vwap.Trade(\"" + watch.stockName + "\", *, *):trade");
	tradeExpr.addMatchListener(calculate);
    }



    /**
     * Implementation of the monitor interface onLoad method. Sets up a top-level 
     * event expression to create new vwap instantiations to calculate VWAP information
     * for a given stock name
     */
    public void onLoad() {
	EventExpression factory = new EventExpression("all com.apama.samples.vwap.VwapWatch(*, >0, *):watch ");
	factory.addMatchListener(new MatchListener() {
				private static final long serialVersionUID = 42L;
                public void match(MatchEvent event) {
		    new Vwap( (VwapWatch)event.getMatchingEvents().get("watch") );
		}
	    });
    } 


    
    // Helper method to calculate the VWAP for the stock when the period has expired.
    // Emits a match if there is a given percentage change (+ or -) in two succesive 
    // non-zero vwaps greater than that requested in the instantiating vwap watch event
    private void periodExpired() {
	// calculate the VWAP (zero if no trade events have been seen)
	if (cumulativeV == 0.0)
	    newVwap = 0.0;
	else
	    newVwap = cumulativePV / cumulativeV;
       	
	// if this VWAP and the last VWAP were non-zero, determine the percentage 
	// change and emit a match if greater than that requested
	if (newVwap > 0.0) {
	    if (lastVwap > 0.0) {
		change = java.lang.Math.abs((lastVwap - newVwap) / lastVwap);
		if (change >= watch.percentageChange) {
		    Match match = new Match(watch.stockName, watch.period, change, lastVwap, newVwap);
		    match.emit();
		}
	    }
	    lastVwap = newVwap;
	}
	
	// reset variables used to accumulate VWAP information
	cumulativeV = 0.0;
	cumulativePV = 0.0;
    }

}







